import { __assign, __awaiter, __generator, __read } from "tslib";
// Import Libs and Handlers
import * as React from 'react';
import { observable, autorun, reaction } from 'mobx';
import { get } from 'lodash-es';
import { app, createPageApi } from '../app/global-api'; // 取对应子包app
import { AppRender } from '../handlers/RenderWrapper';
import { _WEDA_CLOUD_SDK as WEDA_CLOUD_SDK } from '@cloudbase/weda-client';
import { createComputed, createWidgets, retryDataBinds, bindFuncObj, checkAuth, toCssText } from '../utils/index';
import { toCssStyle } from '../utils/adapter/generate';
import { usePageLifeCycle } from '../utils/lifecycle';
import { useScrollTop } from '../utils/hooks';
import { runtime } from '../utils/adapter/index';
import { loadMicroApp } from 'qiankun';
var createDataset = WEDA_CLOUD_SDK.createDataset, createStateDataSourceVar = WEDA_CLOUD_SDK.createStateDataSourceVar, generateParamsParser = WEDA_CLOUD_SDK.generateParamsParser, EXTRA_API = WEDA_CLOUD_SDK.EXTRA_API;
// Import Actions
// Init
export default function App(_a) {
    var page = _a.page, generateComponentsMap = _a.generateComponentsMap, generateActionsMap = _a.generateActionsMap, generateLowcodeModule = _a.generateLowcodeModule, generateWidgetsContext = _a.generateWidgetsContext, generateDataBinds = _a.generateDataBinds, generateComponentSchema = _a.generateComponentSchema, generateContext = _a.generateContext, onRender = _a.onRender, onMounted = _a.onMounted, _b = _a.injectContext, injectContext = _b === void 0 ? {} : _b, _c = _a.initDataContext, initDataContext = _c === void 0 ? {} : _c, _d = _a.options, options = _d === void 0 ? { looseError: false, disabledCheckAuth: false } : _d;
    var _e = __read(React.useState(false), 2), pageContextInited = _e[0], setPageContextInited = _e[1];
    var _f = __read(React.useState(Date.now()), 2), _ = _f[0], setFourceUpdate = _f[1];
    // 数据容器的上下文数据
    var dataContextRef = React.useRef(observable((injectContext === null || injectContext === void 0 ? void 0 : injectContext.dataContext) || initDataContext));
    var dataContext = dataContextRef.current;
    React.useEffect(function () {
        if (injectContext === null || injectContext === void 0 ? void 0 : injectContext.dataContext) {
            dataContextRef.current = observable(injectContext === null || injectContext === void 0 ? void 0 : injectContext.dataContext);
            setFourceUpdate(Date.now());
        }
    }, [injectContext === null || injectContext === void 0 ? void 0 : injectContext.dataContext]);
    // 兼容 this.state / $page.state 两种模式
    var pageCodeContextRef = React.useRef((injectContext === null || injectContext === void 0 ? void 0 : injectContext.$page) || createPageApi());
    var $page = pageCodeContextRef.current;
    var lowcode = generateLowcodeModule();
    var componentSchema, widgetsContext, databinds;
    React.useEffect(function () {
        if (!(injectContext === null || injectContext === void 0 ? void 0 : injectContext.$page)) {
            var dataset = createDataset(page.id);
            var state = observable(lowcode.state || {});
            state.dataset = dataset;
            Object.assign($page, {
                id: page.id,
                state: state,
                computed: createComputed(lowcode.computed || {}, pageCodeContextRef.current),
                handler: bindFuncObj(lowcode.handler || {}, pageCodeContextRef.current),
                randerer: lowcode.renderer,
                dataset: dataset,
            });
        }
        setPageContextInited(true);
    }, []);
    if (pageContextInited) {
        var globalCtx = {
            $page: $page,
            app: app,
            $app: app,
            actionsMap: (generateActionsMap === null || generateActionsMap === void 0 ? void 0 : generateActionsMap()) || {},
        };
        if (generateContext) {
            var context = generateContext(globalCtx, { items: page.items });
            widgetsContext = context.widgetContext;
            databinds = context.dataBinds;
            componentSchema = context.componentSchema;
            // componentSchema = generateComponentSchema(globalCtx, page);
        }
        else {
            widgetsContext = generateWidgetsContext(page);
            databinds = generateDataBinds(globalCtx, page);
            componentSchema = generateComponentSchema(globalCtx, page);
        }
        // widgets 内的 dataBinds 可能需要关联 widgets，需要重新执行 dataBinds
        $page._dataBinds = databinds;
        $page.widgets = createWidgets(widgetsContext, $page._dataBinds || {}, dataContext);
        retryDataBinds();
    }
    return (pageContextInited && (React.createElement(AppContent, { page: page, lowcode: lowcode, componentSchema: componentSchema, pageCodeContext: pageCodeContextRef.current, virtualFields: generateComponentsMap(), onMounted: onMounted, onRender: onRender, dataContext: dataContext, 
        /**
         * 更新数据容器的上下文的方法
         * 会传递到事件emit的地方，将从组件获取到的数据data赋值给上下文
         * 当组件卸载时，传过来的data为undefined即可
         * {
         *   id1: [{...}],
         *   id2: {...},
         *   id3: undefined,
         *   id4: null,
         *   ...
         * }
         * @param id
         * @param data
         */
        updateContext: function (id, data) {
            var _a;
            if (id && !(injectContext === null || injectContext === void 0 ? void 0 : injectContext.dataContext)) {
                dataContext[id] = { data: data };
            }
            (_a = injectContext === null || injectContext === void 0 ? void 0 : injectContext.onDataContextChange) === null || _a === void 0 ? void 0 : _a.call(injectContext, id, data);
        }, options: options })));
}
function AppContent(_a) {
    var _this = this;
    var _b, _c, _d, _e, _f;
    var page = _a.page, lowcode = _a.lowcode, componentSchema = _a.componentSchema, pageCodeContext = _a.pageCodeContext, virtualFields = _a.virtualFields, _g = _a.onMounted, onMounted = _g === void 0 ? function (props) {
        var _a;
        (_a = props === null || props === void 0 ? void 0 : props.default) === null || _a === void 0 ? void 0 : _a.call(props);
    } : _g, _h = _a.onRender, onRender = _h === void 0 ? function () { } : _h, dataContext = _a.dataContext, updateContext = _a.updateContext, options = _a.options;
    // 检查权限
    var _j = __read(React.useState(false), 2), weDaHasLogin = _j[0], setWeDaHasLogin = _j[1];
    var containerRef = React.useRef(null);
    var microApp = React.useRef(null);
    var pureSrc = (_b = page.attributes) === null || _b === void 0 ? void 0 : _b.src;
    var isPure = !!pureSrc;
    // Web 环境页面级别生命周期
    usePageLifeCycle(__assign(__assign({}, lowcode === null || lowcode === void 0 ? void 0 : lowcode.lifecycle), { beforePageCustomLaunch: function (query) { return __awaiter(_this, void 0, void 0, function () {
            var checkAuthResult, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        onMounted({
                            page: page,
                            default: function () {
                                var _a;
                                var document = (runtime === null || runtime === void 0 ? void 0 : runtime.document) || window.document;
                                document.title = ((_a = page.attributes) === null || _a === void 0 ? void 0 : _a.title) || page.id;
                            },
                        });
                        EXTRA_API.setParams(page.id, query || {});
                        initWatchMethods(pageCodeContext);
                        _a = (options === null || options === void 0 ? void 0 : options.disabledCheckAuth);
                        if (_a) return [3 /*break*/, 2];
                        return [4 /*yield*/, checkAuth(app, app.id, pageCodeContext)];
                    case 1:
                        _a = (_b.sent());
                        _b.label = 2;
                    case 2:
                        checkAuthResult = _a;
                        setWeDaHasLogin(checkAuthResult);
                        createStateDataSourceVar(page.id, generateParamsParser({ app: app, $page: pageCodeContext }));
                        onRender === null || onRender === void 0 ? void 0 : onRender();
                        return [2 /*return*/];
                }
            });
        }); } }), { app: app, page: pageCodeContext, options: { looseError: options === null || options === void 0 ? void 0 : options.looseError } });
    // 切换页面滚动到顶部
    useScrollTop();
    React.useEffect(function () {
        var _a, _b;
        if (isPure) {
            if (weDaHasLogin) {
                if (!microApp.current) {
                    microApp.current = loadMicroApp({
                        name: page.id,
                        entry: pureSrc,
                        container: containerRef.current,
                    }, {
                        fetch: runtime.fetch,
                    });
                }
                else {
                    (_b = (_a = microApp.current).mount) === null || _b === void 0 ? void 0 : _b.call(_a);
                }
            }
            else if (microApp.current) {
                microApp.current.unmount();
            }
        }
    }, [weDaHasLogin]);
    React.useEffect(function () {
        app.__internal__.activePage = pageCodeContext;
        return function () {
            if (microApp.current) {
                microApp.current.unmount();
            }
        };
    }, []);
    return (React.createElement("div", { ref: containerRef, className: "weapps-page weapps-page-" + page.id },
        React.createElement("style", { type: "text/css", dangerouslySetInnerHTML: {
                __html: "" + toCssText(((_c = page.attributes) === null || _c === void 0 ? void 0 : _c.style) || toCssStyle(((_d = page === null || page === void 0 ? void 0 : page.extra) === null || _d === void 0 ? void 0 : _d.commonStyle) || {}), 'body') + (lowcode.style || [])
                    .map(function (item) { return item.code; })
                    .join('\n'),
            } }),
        weDaHasLogin &&
            !isPure &&
            (((_e = lowcode.lifecycle) === null || _e === void 0 ? void 0 : _e.onPageRender) ? ((_f = lowcode.lifecycle) === null || _f === void 0 ? void 0 : _f.onPageRender()) : (React.createElement(AppRender, { virtualFields: virtualFields, componentSchema: componentSchema, codeContext: pageCodeContext, dataContext: dataContext, updateContext: updateContext })))));
}
function initWatchMethods(pageContext) {
    // const { watch = {}, watchState = {}, watchWidget = {}, watchEffects = {}} = pageAPI
    var _a = {}, _b = _a.watch, watch = _b === void 0 ? {} : _b, _c = _a.watchState, watchState = _c === void 0 ? {} : _c, _d = _a.watchWidget, watchWidget = _d === void 0 ? {} : _d, _e = _a.watchEffects, watchEffects = _e === void 0 ? {} : _e;
    // # watch effect
    var watchEffectDisposers = Object.keys(watchEffects).map(function (fnName) {
        return autorun(watchEffects[fnName].bind(pageContext));
    });
    var disposers = watchEffectDisposers;
    // # watch state
    Object.keys(watchState).map(function (key) { return runWatcher(parseWatcher(watchState[key]), pageContext.state, key, 'watchState'); });
    // # watch widgets
    Object.keys(watchWidget).map(function (key) {
        return runWatcher(parseWatcher(watchWidget[key]), pageContext.widgets, key, 'watchWidgets');
    });
    function runWatcher(_a, target, key, label) {
        var _b = _a === void 0 ? {} : _a, handler = _b.handler, immediate = _b.immediate;
        if (!handler) {
            console.error("Invalid " + label + "(" + key + ") of " + pageContext.node.widgetType + ", watch must a function or {handler: function}");
            return;
        }
        var disposer = reaction(function () { return get(target, key); }, handler.bind(pageContext), { fireImmediately: immediate });
        disposers.push(disposer);
    }
}
function parseWatcher(watcher) {
    if (!watcher)
        return;
    if (watcher instanceof Function) {
        return { handler: watcher, immediate: false };
    }
    var handler = watcher.handler, _a = watcher.immediate, immediate = _a === void 0 ? false : _a;
    if (!(handler instanceof Function))
        return;
    return { handler: handler, immediate: immediate };
}
//# sourceMappingURL=index.js.map