import { __assign, __awaiter, __generator, __read, __rest, __spreadArray } from "tslib";
import { widgetKeys } from './widgets';
import { app as _app } from '../app/global-api';
import { auth } from '@cloudbase/weda-client';
var loginScope = auth.loginScope, getAccessToken = auth.getAccessToken;
/**
 * All data bindings are generated as functions: (forItems) => any
 * @param {*} dataBinds
 * @param {*} forItems
 */
export function resolveDataBinds(dataBinds, forItems, codeContext, scopeContext, throwError, dataContext) {
    var resolvedProps = {};
    for (var prop in dataBinds) {
        var fn = dataBinds[prop];
        try {
            if (codeContext === null || codeContext === void 0 ? void 0 : codeContext.$WEAPPS_COMP) {
                fn = fn.bind(codeContext.$WEAPPS_COMP);
            }
            resolvedProps[prop] = fn(forItems, codeContext === null || codeContext === void 0 ? void 0 : codeContext.event, dataContext);
        }
        catch (e) {
            console.error('Error resolving data binding', prop, dataBinds[prop], e);
            if (throwError) {
                throw e;
            }
        }
    }
    return resolvedProps;
}
/**
 * 对函数进行批量绑定
 * @param {*} funcObj
 * @param {*} context
 */
export function bindFuncObj(funcObj, context) {
    if (funcObj === void 0) { funcObj = {}; }
    return Object.keys(funcObj).reduce(function (obj, fnName) {
        var _a, _b;
        obj[fnName] = (_b = (_a = funcObj[fnName]) === null || _a === void 0 ? void 0 : _a.bind) === null || _b === void 0 ? void 0 : _b.call(_a, context);
        return obj;
    }, {});
}
export function kebabCase(str) {
    return str.replace(/[A-Z]/g, function (match) {
        return "-" + match.toLowerCase();
    });
}
export function camelcase(str, firstUpperCase) {
    if (firstUpperCase === void 0) { firstUpperCase = false; }
    str = str.replace(/[_-]([a-z])/g, function (l) {
        return l[1].toUpperCase();
    });
    if (firstUpperCase)
        str = str.charAt(0).toUpperCase() + str.slice(1);
    return str;
}
export var isEmptyObj = function (obj) {
    if (!isPlainObject(obj)) {
        return false;
    }
    for (var i in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, i)) {
            return false;
        }
    }
    return true;
};
export function isPlainObject(src) {
    return Object.prototype.toString.call(src) === '[object Object]';
}
/**
 * 用于处理自定义组件props传参结构，对系统变量进行保留
 */
export function resolveComponentProps(props, isPlainProps) {
    var _a, _b;
    var _c = (_app || {}).__internal__, __internal__ = _c === void 0 ? {} : _c;
    var resolveStaticResourceUrl = (__internal__ || {}).resolveStaticResourceUrl;
    var staticResourceAttribute = props.staticResourceAttribute;
    (_a = staticResourceAttribute === null || staticResourceAttribute === void 0 ? void 0 : staticResourceAttribute.map) === null || _a === void 0 ? void 0 : _a.call(staticResourceAttribute, function (property) {
        var _a;
        if ((_a = props.data) === null || _a === void 0 ? void 0 : _a[property]) {
            props.data[property] = resolveStaticResourceUrl(props.data[property]);
        }
    });
    if (!isPlainProps) {
        return __assign({}, props);
    }
    var _d = props.data, data = _d === void 0 ? {} : _d, _e = props.events, events = _e === void 0 ? [] : _e, restProps = __rest(props, ["data", "events"]);
    var customProps = __assign({}, data);
    var builtinProps = __spreadArray(__spreadArray([
        // react 保留字
        'ref',
        'key',
        'dangerouslySetInnerHTML',
        'className',
        'htmlFor',
        'style',
        'contentEditable',
        // lowcode 保留字
        'events',
        'children',
        '_parentId',
        '_visible',
        '_listeners'
    ], __read(widgetKeys), false), [
        'classList',
        'widgetType',
        'staticResourceAttribute',
        // 小程序保留字
        'id',
        'class',
        'hidden',
        'slot',
    ], false);
    // delete builtin props
    builtinProps.map(function (prop) { return delete customProps[prop]; });
    // 选区events处理
    var _selectableBlockEvents = customProps._selectableBlockEvents, restData = __rest(customProps, ["_selectableBlockEvents"]);
    var _selectableBlockEmit = _selectableBlockEvents.emit;
    var selectableBlockEvents = ((_b = _selectableBlockEvents === null || _selectableBlockEvents === void 0 ? void 0 : _selectableBlockEvents.events) === null || _b === void 0 ? void 0 : _b.reduce(function (events, item) {
        var propName = item;
        events[propName] = function (e) { return _selectableBlockEmit(propName, e); };
        return events;
    }, {})) || {};
    try {
        _selectableBlockEvents.events = new Proxy(selectableBlockEvents, {
            get: function (obj, prop) {
                return prop in obj ? obj[prop] : function (e) { return _selectableBlockEmit(prop, e); };
            },
        });
    }
    catch (e) {
        console.warn('selectableBlockEvents', selectableBlockEvents);
    }
    return __assign(__assign(__assign({}, restData), restProps), { _selectableBlockEvents: _selectableBlockEvents, events: new Proxy(events.reduce(function (events, item) {
            var propName = item;
            events[propName] = function (e) { return restProps.emit(propName, e); };
            return events;
        }, {}), {
            get: function (obj, prop) {
                return prop in obj ? obj[prop] : function (e) { return restProps.emit(prop, e); };
            },
        }) });
}
var SCOPE_SLOT_COMPONENT_LIB = ['tea_basis', 'tea_shop', 'crm_basis'];
var SCOPE_SLOT_MAP = SCOPE_SLOT_COMPONENT_LIB.reduce(function (map, lib) {
    map[lib + ":TableMatching"] = {
        headerSlot: true,
        recordSlot: true,
    };
    map[lib + ":TableExpanded"] = {
        headerSlot: true,
        recordSlot: true,
    };
    map[lib + ":TabsTable"] = {
        headerSlot: true,
        recordSlot: true,
    };
    map[lib + ":Table"] = {
        headerSlot: true,
        recordSlot: true,
    };
    return map;
}, {});
export function isSlot(comp) {
    return !comp['module'] && !comp.component;
}
export function isScopeSlot(comp, slot) {
    var sourceKey = comp.module && comp.component && comp.module + ":" + comp.component;
    var map = SCOPE_SLOT_MAP[sourceKey];
    return map === null || map === void 0 ? void 0 : map[slot];
}
export function findLoginPage(app) {
    if (app === void 0) { app = _app; }
    var _a = app.__internal__.getConfig().pages, pages = _a === void 0 ? [] : _a;
    return pages.find(function (item) { return item.type === 'login'; });
}
var _AUTH_CONFIG_CACHE = null;
export function getAuthConfig(app) {
    if (app === void 0) { app = _app; }
    return __awaiter(this, void 0, void 0, function () {
        var res, settingData_1, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (_AUTH_CONFIG_CACHE) {
                        return [2 /*return*/, _AUTH_CONFIG_CACHE];
                    }
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, app.cloud.callWedaApi({
                            action: 'DescribeRuntimeResourceStrategy',
                            data: {
                                ResourceType: 'app',
                                ResourceId: app.id,
                            },
                        })];
                case 2:
                    res = _a.sent();
                    settingData_1 = {};
                    // 云api不支持map只能传字符串，需要转换
                    res.forEach(function (item) {
                        settingData_1[item.Key] = ['AllowRegister', 'NeedLogin'].includes(item.Key) ? item.Value === '1' : item.Value;
                    });
                    _AUTH_CONFIG_CACHE = settingData_1;
                    return [2 /*return*/, _AUTH_CONFIG_CACHE];
                case 3:
                    e_1 = _a.sent();
                    return [2 /*return*/, {
                            NeedLogin: false,
                            RejectStrategy: 'show_warning',
                        }];
                case 4: return [2 /*return*/];
            }
        });
    });
}
var _AUTH_CACHE_MAP = {};
function getAccessPermission(app, appId, pageId) {
    return __awaiter(this, void 0, void 0, function () {
        var cacheKey, isAccess, res, e_2;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    cacheKey = appId + "-" + pageId;
                    if (_AUTH_CACHE_MAP[cacheKey] !== undefined) {
                        return [2 /*return*/, _AUTH_CACHE_MAP[cacheKey]];
                    }
                    isAccess = false;
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, app.cloud.callWedaApi({
                            action: 'DescribeResourcesPermission',
                            data: {
                                ResourceType: "app",
                                ResourceIdList: [cacheKey],
                                AppResourceId: appId,
                            },
                        })];
                case 2:
                    res = _a.sent();
                    if (Array.isArray(res) && res.length > 0) {
                        isAccess = !!res[0].IsAccess;
                    }
                    _AUTH_CACHE_MAP[cacheKey] = isAccess;
                    return [3 /*break*/, 4];
                case 3:
                    e_2 = _a.sent();
                    console.warn('getAccessPermission', e_2);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/, isAccess];
            }
        });
    });
}
export function checkAnonymous() {
    return __awaiter(this, void 0, void 0, function () {
        var isAnonymous, _a, scope, accessToken, e_3;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    isAnonymous = true;
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, Promise.all([loginScope(), getAccessToken()])];
                case 2:
                    _a = __read.apply(void 0, [_b.sent(), 2]), scope = _a[0], accessToken = _a[1].accessToken;
                    if (accessToken && scope !== 'anonymous') {
                        isAnonymous = false;
                    }
                    return [3 /*break*/, 4];
                case 3:
                    e_3 = _b.sent();
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/, isAnonymous];
            }
        });
    });
}
var loginConfig;
export function getLoginConfig(app) {
    if (app === void 0) { app = _app; }
    return __awaiter(this, void 0, void 0, function () {
        var _a, staticResourceDomain, id, url, response, e_4;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    if (loginConfig !== undefined) {
                        return [2 /*return*/, loginConfig];
                    }
                    _a = app.__internal__.getConfig(), staticResourceDomain = _a.staticResourceDomain, id = _a.id;
                    url = "https://" + staticResourceDomain + "/" + id + "/login.config.json?t=" + Date.now();
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 4, , 5]);
                    return [4 /*yield*/, fetch(url)];
                case 2:
                    response = _b.sent();
                    return [4 /*yield*/, response.json()];
                case 3:
                    loginConfig = _b.sent();
                    return [3 /*break*/, 5];
                case 4:
                    e_4 = _b.sent();
                    loginConfig = {};
                    console.error("\u52A0\u8F7D\u767B\u5F55\u914D\u7F6E" + url + "\u5931\u8D25", e_4);
                    return [3 /*break*/, 5];
                case 5: return [2 /*return*/, loginConfig];
            }
        });
    });
}
/**
 * 检查页面权限
 **/
export function checkAuth(app, appId, $page) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function () {
        var loginConfig, requestList, _c, isAccess, authConfig, isAnonymous;
        return __generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    app.showNavigationBarLoading();
                    return [4 /*yield*/, getLoginConfig()];
                case 1:
                    loginConfig = _d.sent();
                    requestList = [getAccessPermission(app, appId, $page.id)];
                    // 登录配置内有web登录项则认为开启web登录
                    if (((_a = loginConfig.web) === null || _a === void 0 ? void 0 : _a.length) > 0) {
                        requestList.push(getAuthConfig(app));
                        requestList.push(checkAnonymous());
                    }
                    return [4 /*yield*/, Promise.all(requestList)];
                case 2:
                    _c = __read.apply(void 0, [_d.sent(), 3]), isAccess = _c[0], authConfig = _c[1], isAnonymous = _c[2];
                    app.hideNavigationBarLoading();
                    if (((_b = loginConfig.web) === null || _b === void 0 ? void 0 : _b.length) > 0 && isAnonymous) {
                        if (authConfig.NeedLogin || (!isAccess && authConfig.RejectStrategy === 'to_login')) {
                            redirectToLogin($page);
                            return [2 /*return*/, false];
                        }
                    }
                    if (!isAccess) {
                        app.showToast({
                            title: '页面无访问权限',
                            icon: 'error',
                        });
                    }
                    return [2 /*return*/, isAccess];
            }
        });
    });
}
export function redirectToLogin(currentPage) {
    var _a;
    return __awaiter(this, void 0, void 0, function () {
        var app, loginConfig, _b, staticResourceDomain, envId, id, clientId, url;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    // 去登录则清空权限缓存。
                    _AUTH_CACHE_MAP = {};
                    app = _app;
                    return [4 /*yield*/, getLoginConfig()];
                case 1:
                    loginConfig = _c.sent();
                    if (((_a = loginConfig.web) === null || _a === void 0 ? void 0 : _a.length) > 0) {
                        _b = app.__internal__.getConfig(), staticResourceDomain = _b.staticResourceDomain, envId = _b.envId, id = _b.id, clientId = _b.clientId;
                        url = location.origin + "/__auth/?redirect_uri=" + encodeURIComponent(location.href) + "&app_id=" + id + "&env_id=" + envId + "&client_id=" + clientId + "&s_domain=" + staticResourceDomain;
                        app.redirectTo({
                            mode: 'web',
                            params: {},
                            url: url,
                        });
                    }
                    else {
                        app.showToast({
                            title: '用户未登录',
                            icon: 'error',
                        });
                    }
                    return [2 /*return*/];
            }
        });
    });
}
//# sourceMappingURL=common.js.map